package controllers

import (
	"Esan-Shabu/configs"
	"Esan-Shabu/models"
	"Esan-Shabu/responses"
	"context"
	"net/http"
	"time"

	"github.com/gofiber/fiber/v2"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo"
)

var promotionCollection *mongo.Collection = configs.GetCollection(configs.DB, "promotions")

func CreatePromotion(c *fiber.Ctx) error {
	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
	var promotion models.Promotion
	defer cancel()

	//validate the request body
	if err := c.BodyParser(&promotion); err != nil {
		return c.Status(http.StatusBadRequest).JSON(responses.PromotionResponse{Status: http.StatusBadRequest, Message: "error", Data: &fiber.Map{"data": err.Error()}})
	}

	//use the validator library to validate required fields
	if validationErr := validate.Struct(&promotion); validationErr != nil {
		return c.Status(http.StatusBadRequest).JSON(responses.PromotionResponse{Status: http.StatusBadRequest, Message: "error", Data: &fiber.Map{"data": validationErr.Error()}})
	}

	newPromotion := models.Promotion{
		Name:  promotion.Name,
		Image: promotion.Image,
	}

	result, err := promotionCollection.InsertOne(ctx, newPromotion)
	if err != nil {
		return c.Status(http.StatusInternalServerError).JSON(responses.PromotionResponse{Status: http.StatusInternalServerError, Message: "error", Data: &fiber.Map{"data": err.Error()}})
	}

	return c.Status(http.StatusCreated).JSON(responses.PromotionResponse{Status: http.StatusCreated, Message: "success", Data: &fiber.Map{"data": result}})
}

func GetAllPromotions(c *fiber.Ctx) error {
	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
	var promotions []models.Promotion
	defer cancel()

	results, err := promotionCollection.Find(ctx, bson.M{})

	if err != nil {
		return c.Status(http.StatusInternalServerError).JSON(responses.PromotionResponse{Status: http.StatusInternalServerError, Message: "error", Data: &fiber.Map{"data": err.Error()}})
	}

	//reading from the db in an optimal way
	defer results.Close(ctx)
	for results.Next(ctx) {
		var singleUser models.Promotion
		if err = results.Decode(&singleUser); err != nil {
			return c.Status(http.StatusInternalServerError).JSON(responses.PromotionResponse{Status: http.StatusInternalServerError, Message: "error", Data: &fiber.Map{"data": err.Error()}})
		}

		promotions = append(promotions, singleUser)
	}

	return c.Status(http.StatusOK).JSON(
		responses.PromotionResponse{Status: http.StatusOK, Message: "success", Data: &fiber.Map{"data": promotions}},
	)
}

func DeleteAPromotion(c *fiber.Ctx) error {
	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
	promotionId := c.Params("promotionId")
	defer cancel()

	objId, _ := primitive.ObjectIDFromHex(promotionId)

	result, err := promotionCollection.DeleteOne(ctx, bson.M{"_id": objId})
	if err != nil {
		return c.Status(http.StatusInternalServerError).JSON(responses.PromotionResponse{Status: http.StatusInternalServerError, Message: "error", Data: &fiber.Map{"data": err.Error()}})
	}

	if result.DeletedCount < 1 {
		return c.Status(http.StatusNotFound).JSON(
			responses.PromotionResponse{Status: http.StatusNotFound, Message: "error", Data: &fiber.Map{"data": "Promotion with specified ID not found!"}},
		)
	}

	return c.Status(http.StatusOK).JSON(
		responses.PromotionResponse{Status: http.StatusOK, Message: "success", Data: &fiber.Map{"data": "Promotion successfully deleted!"}},
	)
}
